@@ -5,7 +5,10 @@ import os
 import tempfile
 import warnings
 from pathlib import Path
+from typing import Any
+from typing import Dict
 from typing import List
+from typing import Tuple
 
 import pluggy
 from _pytask.shared import convert_truthy_or_falsy_to_bool
@@ -18,7 +21,7 @@ from _pytask.shared import to_list
 hookimpl = pluggy.HookimplMarker("pytask")
 
 
-_IGNORED_FOLDERS = [
+_IGNORED_FOLDERS: List[str] = [
     ".git/*",
     ".hg/*",
     ".svn/*",
@@ -26,7 +29,7 @@ _IGNORED_FOLDERS = [
 ]
 
 
-_IGNORED_FILES = [
+_IGNORED_FILES: List[str] = [
     ".codecov.yml",
     ".gitignore",
     ".pre-commit-config.yaml",
@@ -41,10 +44,10 @@ _IGNORED_FILES = [
 ]
 
 
-_IGNORED_FILES_AND_FOLDERS = _IGNORED_FILES + _IGNORED_FOLDERS
+_IGNORED_FILES_AND_FOLDERS: List[str] = _IGNORED_FILES + _IGNORED_FOLDERS
 
 
-IGNORED_TEMPORARY_FILES_AND_FOLDERS = [
+IGNORED_TEMPORARY_FILES_AND_FOLDERS: List[str] = [
     "*.egg-info/*",
     ".ipynb_checkpoints/*",
     ".mypy_cache/*",
@@ -68,7 +71,9 @@ IS_FILE_SYSTEM_CASE_SENSITIVE = is_file_system_case_sensitive()
 
 
 @hookimpl
-def pytask_configure(pm, config_from_cli):
+def pytask_configure(
+    pm: pluggy.PluginManager, config_from_cli: Dict[str, Any]
+) -> Dict[str, Any]:
     """Configure pytask."""
     config = {"pm": pm}
 
@@ -91,7 +96,9 @@ def pytask_configure(pm, config_from_cli):
 
     # If paths are set in the configuration, process them.
     if config_from_file.get("paths"):
-        paths_from_file = parse_value_or_multiline_option(config_from_file.get("paths"))
+        paths_from_file = to_list(
+            parse_value_or_multiline_option(config_from_file.get("paths"))
+        )
         config_from_file["paths"] = [
             config["config"].parent.joinpath(p).resolve() for p in paths_from_file
         ]
@@ -123,7 +130,11 @@ def pytask_configure(pm, config_from_cli):
 
 
 @hookimpl
-def pytask_parse_config(config, config_from_cli, config_from_file):
+def pytask_parse_config(
+    config: Dict[str, Any],
+    config_from_cli: Dict[str, Any],
+    config_from_file: Dict[str, Any],
+) -> None:
     """Parse the configuration."""
     config["command"] = config_from_cli.get("command", "build")
 
@@ -199,12 +210,12 @@ def pytask_parse_config(config, config_from_cli, config_from_file):
 
 
 @hookimpl
-def pytask_post_parse(config):
+def pytask_post_parse(config: Dict[str, Any]) -> None:
     """Sort markers alphabetically."""
     config["markers"] = {k: config["markers"][k] for k in sorted(config["markers"])}
 
 
-def _find_project_root_and_ini(paths: List[Path]):
+def _find_project_root_and_ini(paths: List[Path]) -> Tuple[Path, Path]:
     """Find the project root and configuration file from a list of paths."""
     try:
         common_ancestor = Path(os.path.commonpath(paths))
@@ -243,7 +254,7 @@ def _find_project_root_and_ini(paths: List[Path]):
     return root, config_path
 
 
-def _read_config(path):
+def _read_config(path: Path) -> Dict[str, Any]:
     """Read the configuration from a file with a [pytask] section."""
     config = configparser.ConfigParser()
     config.read(path)
