@@ -1,36 +1,43 @@
+"""This module contains hook implementations concerning the execution."""
 import sys
 import time
+from typing import Any
+from typing import Dict
+from typing import List
 
 from _pytask.config import hookimpl
 from _pytask.console import console
+from _pytask.console import create_summary_panel
+from _pytask.console import create_url_style_for_task
+from _pytask.console import format_task_id
+from _pytask.console import unify_styles
 from _pytask.dag import descending_tasks
-from _pytask.dag import node_and_neighbors
 from _pytask.dag import TopologicalSorter
-from _pytask.database import create_or_update_state
-from _pytask.enums import ColorCode
+from _pytask.database import update_states_in_database
 from _pytask.exceptions import ExecutionError
 from _pytask.exceptions import NodeNotFoundError
 from _pytask.mark import Mark
 from _pytask.nodes import FilePathNode
+from _pytask.nodes import MetaTask
+from _pytask.outcomes import count_outcomes
 from _pytask.outcomes import Exit
-from _pytask.outcomes import Persisted
-from _pytask.outcomes import Skipped
+from _pytask.outcomes import TaskOutcome
 from _pytask.report import ExecutionReport
+from _pytask.session import Session
 from _pytask.shared import get_first_non_none_value
-from _pytask.shared import reduce_node_name
 from _pytask.traceback import format_exception_without_traceback
 from _pytask.traceback import remove_traceback_from_exc_info
 from _pytask.traceback import render_exc_info
+from rich.text import Text
 
 
 @hookimpl
-def pytask_post_parse(config):
-    if config["show_errors_immediately"]:
-        config["pm"].register(ShowErrorsImmediatelyPlugin)
-
-
-@hookimpl
-def pytask_parse_config(config, config_from_cli, config_from_file):
+def pytask_parse_config(
+    config: Dict[str, Any],
+    config_from_cli: Dict[str, Any],
+    config_from_file: Dict[str, Any],
+) -> None:
+    """Parse the configuration."""
     config["show_errors_immediately"] = get_first_non_none_value(
         config_from_cli,
         config_from_file,
@@ -41,7 +48,14 @@ def pytask_parse_config(config, config_from_cli, config_from_file):
 
 
 @hookimpl
-def pytask_execute(session):
+def pytask_post_parse(config: Dict[str, Any]) -> None:
+    """Adjust the configuration after intermediate values have been parsed."""
+    if config["show_errors_immediately"]:
+        config["pm"].register(ShowErrorsImmediatelyPlugin)
+
+
+@hookimpl
+def pytask_execute(session: Session) -> None:
     """Execute tasks."""
     session.hook.pytask_execute_log_start(session=session)
     session.scheduler = session.hook.pytask_execute_create_scheduler(session=session)
@@ -52,7 +66,7 @@ def pytask_execute(session):
 
 
 @hookimpl
-def pytask_execute_log_start(session):
+def pytask_execute_log_start(session: Session) -> None:
     """Start logging."""
     session.execution_start = time.time()
 
@@ -61,7 +75,7 @@ def pytask_execute_log_start(session):
 
 
 @hookimpl(trylast=True)
-def pytask_execute_create_scheduler(session):
+def pytask_execute_create_scheduler(session: Session) -> TopologicalSorter:
     """Create a scheduler based on topological sorting."""
     scheduler = TopologicalSorter.from_dag(session.dag)
     scheduler.prepare()
@@ -69,21 +83,25 @@ def pytask_execute_create_scheduler(session):
 
 
 @hookimpl
-def pytask_execute_build(session):
+def pytask_execute_build(session: Session) -> bool:
     """Execute tasks."""
-    for name in session.scheduler.static_order():
-        task = session.dag.nodes[name]["task"]
-        report = session.hook.pytask_execute_task_protocol(session=session, task=task)
-        session.execution_reports.append(report)
+    if isinstance(session.scheduler, TopologicalSorter):
+        for name in session.scheduler.static_order():
+            task = session.dag.nodes[name]["task"]
+            report = session.hook.pytask_execute_task_protocol(
+                session=session, task=task
+            )
+            session.execution_reports.append(report)
 
-        if session.should_stop:
-            return True
-
-    return True
+            if session.should_stop:
+                return True
+        return True
+    else:
+        return None
 
 
 @hookimpl
-def pytask_execute_task_protocol(session, task):
+def pytask_execute_task_protocol(session: Session, task: MetaTask) -> ExecutionReport:
     """Follow the protocol to execute each task."""
     session.hook.pytask_execute_task_log_start(session=session, task=task)
     try:
@@ -105,7 +123,7 @@ def pytask_execute_task_protocol(session, task):
 
 
 @hookimpl(trylast=True)
-def pytask_execute_task_setup(session, task):
+def pytask_execute_task_setup(session: Session, task: MetaTask) -> None:
     """Set up the execution of a task.
 
     1. Check whether all dependencies of a task are available.
@@ -130,13 +148,13 @@ def pytask_execute_task_setup(session, task):
 
 
 @hookimpl
-def pytask_execute_task(task):
+def pytask_execute_task(task: MetaTask) -> None:
     """Execute task."""
     task.execute()
 
 
 @hookimpl
-def pytask_execute_task_teardown(session, task):
+def pytask_execute_task_teardown(session: Session, task: MetaTask) -> None:
     """Check if each produced node was indeed produced."""
     for product in session.dag.successors(task.name):
         node = session.dag.nodes[product]["node"]
@@ -149,7 +167,9 @@ def pytask_execute_task_teardown(session, task):
 
 
 @hookimpl(trylast=True)
-def pytask_execute_task_process_report(session, report):
+def pytask_execute_task_process_report(
+    session: Session, report: ExecutionReport
+) -> bool:
     """Process the execution report of a task.
 
     If a task failed, skip all subsequent tasks. Else, update the states of related
@@ -157,20 +177,16 @@ def pytask_execute_task_process_report(session, report):
 
     """
     task = report.task
-    if report.success:
-        _update_states_in_database(session.dag, task.name)
-        report.symbol = "."
-        report.color = ColorCode.SUCCESS
+    if report.outcome == TaskOutcome.SUCCESS:
+        update_states_in_database(session.dag, task.name)
     else:
-        report.symbol = "F"
-        report.color = ColorCode.FAILED
         for descending_task_name in descending_tasks(task.name, session.dag):
             descending_task = session.dag.nodes[descending_task_name]["task"]
             descending_task.markers.append(
                 Mark(
                     "skip_ancestor_failed",
                     (),
-                    {"reason": f"Previous task '{task.name}' failed."},
+                    {"reason": f"Previous task {task.name!r} failed."},
                 )
             )
 
@@ -185,67 +201,72 @@ def pytask_execute_task_process_report(session, report):
 
 
 @hookimpl(trylast=True)
-def pytask_execute_task_log_end(report):
+def pytask_execute_task_log_end(session: Session, report: ExecutionReport) -> None:
     """Log task outcome."""
-    console.print(report.symbol, style=report.color, end="")
+    url_style = create_url_style_for_task(
+        report.task.function, session.config["editor_url_scheme"]
+    )
+    console.print(
+        report.outcome.symbol,
+        style=unify_styles(report.outcome.style, url_style),
+        end="",
+    )
 
 
 class ShowErrorsImmediatelyPlugin:
     @staticmethod
     @hookimpl(tryfirst=True)
-    def pytask_execute_task_log_end(session, report):
-        if not report.success:
+    def pytask_execute_task_log_end(session: Session, report: ExecutionReport) -> None:
+        if report.outcome == TaskOutcome.FAIL:
             _print_errored_task_report(session, report)
 
 
 @hookimpl
-def pytask_execute_log_end(session, reports):
+def pytask_execute_log_end(session: Session, reports: List[ExecutionReport]) -> bool:
+    """Log information on the execution."""
     session.execution_end = time.time()
 
-    n_failed = len(reports) - sum(report.success for report in reports)
-    n_skipped = sum(
-        isinstance(report.exc_info[1], Skipped) for report in reports if report.exc_info
-    )
-    n_persisted = sum(
-        isinstance(report.exc_info[1], Persisted)
-        for report in reports
-        if report.exc_info
-    )
-    n_successful = len(reports) - n_failed - n_skipped - n_persisted
+    counts = count_outcomes(reports, TaskOutcome)
 
-    console.print()
-    if n_failed:
-        console.rule(f"[{ColorCode.FAILED}]Failures", style=ColorCode.FAILED)
+    if session.config["show_traceback"] != "no":
         console.print()
+        if counts[TaskOutcome.FAIL]:
+            console.rule(
+                Text("Failures", style=TaskOutcome.FAIL.style),
+                style=TaskOutcome.FAIL.style,
+            )
+            console.print()
 
-    for report in reports:
-        if not report.success:
-            _print_errored_task_report(session, report)
+        for report in reports:
+            if report.outcome in (TaskOutcome.FAIL, TaskOutcome.SKIP_PREVIOUS_FAILED):
+                _print_errored_task_report(session, report)
+
+    console.rule(style="dim")
+
+    panel = create_summary_panel(counts, TaskOutcome, "Collected tasks")
+    console.print(panel)
 
     session.hook.pytask_log_session_footer(
         session=session,
-        infos=[
-            (n_successful, "succeeded", ColorCode.SUCCESS),
-            (n_persisted, "persisted", ColorCode.SUCCESS),
-            (n_failed, "failed", ColorCode.FAILED),
-            (n_skipped, "skipped", ColorCode.SKIPPED),
-        ],
-        duration=round(session.execution_end - session.execution_start, 2),
-        color=ColorCode.FAILED if n_failed else ColorCode.SUCCESS,
+        duration=session.execution_end - session.execution_start,
+        outcome=TaskOutcome.FAIL if counts[TaskOutcome.FAIL] else TaskOutcome.SUCCESS,
     )
 
-    if n_failed:
+    if counts[TaskOutcome.FAIL]:
         raise ExecutionError
 
     return True
 
 
-def _print_errored_task_report(session, report):
+def _print_errored_task_report(session: Session, report: ExecutionReport) -> None:
     """Print the traceback and the exception of an errored report."""
-    task_name = reduce_node_name(report.task, session.config["paths"])
-    if len(task_name) > console.width - 15:
-        task_name = report.task.base_name
-    console.rule(f"[{ColorCode.FAILED}]Task {task_name} failed", style=ColorCode.FAILED)
+    task_name = format_task_id(
+        task=report.task,
+        editor_url_scheme=session.config["editor_url_scheme"],
+        short_name=True,
+    )
+    text = Text.assemble("Task ", task_name, " failed", style="failed")
+    console.rule(text, style=report.outcome.style)
 
     console.print()
 
@@ -260,10 +281,3 @@ def _print_errored_task_report(session, report):
         if key in ("stdout", "stderr") and show_capture in (key, "all"):
             console.rule(f"Captured {key} during {when}", style=None)
             console.print(content)
-
-
-def _update_states_in_database(dag, task_name):
-    """Update the state for each node of a task in the database."""
-    for name in node_and_neighbors(dag, task_name):
-        node = dag.nodes[name].get("task") or dag.nodes[name]["node"]
-        create_or_update_state(task_name, node.name, node.state())
